#########################################################################
#
# playedit for tk3play 1.00 last edit: 04/22/97
#  
#  Copyright (C) 1997 Brian Foutz (bef2@cornell.edu)
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#
#
########################################################################

set PlayEdit(listfont) -adobe-helvetica-bold-r-*-*-12-*-*-*-*-*-*-*
set PlayEdit(entryfont) -adobe-helvetica-bold-r-*-*-12-*-*-*-*-*-*-*
set PlayEdit(bgcolor) black 
set PlayEdit(fgcolor) green 
set PlayEdit(fghighlight) orange 
set PlayEdit(bghighlight) midnightblue 
set PlayEdit(crcolor) white

set PlayEdit(fchoices) {{*.mp*} {*} {*.mp2} {*.mp3}}

#
# URL Support Procedure by Saturn
#
#proc IsURL { URL } {
#  if [regexp "http://" $URL] then {
#    return 1
#  } else {
#    return 0 
#  }
#}

#
# FileList procedures
#

proc FileList(select) { } {
   global FileList
   set sel [$FileList(filelist) curselection]
   set first [lindex $sel 0]
   set FileList(filename) [$FileList(filelist) get $first]
#   set FileList(filename) [$FileList(filelist) get active]
}

proc FileList(enter) { } {
   global FileList DirList

   if {![check_file $DirList(directory)$FileList(filename)]} {return}

   # Find entered filename in the filelist
   set index [lsearch [$FileList(filelist) get 0 end] $FileList(filename)]
   $FileList(filelist) selection clear 0 end
   $FileList(filelist) selection set $index
   $FileList(filelist) selection anchor $index
   return
}

proc FileList(update) {dirname} {
   global FileList

   $FileList(filelist) delete 0 end
   foreach i [lsort [glob -nocomplain $dirname/$FileList(filter) \
                                      $dirname/.$FileList(filter)]] {
      if {![file isdirectory $i]} {
         $FileList(filelist) insert end [file tail $i]
      }
   }

   # blank out entry window
   set FileList(filename) ""
   return
}

#
# DirList procedures
#

proc DirList(select) { } {
   global DirList
   set sel [$DirList(dirlist) curselection]
   set dirname [lindex $DirList(dirname_list) $sel]
   set DirList(directory) $dirname
   DirList(update) $dirname
   return
}

proc DirList(append) { x y } {
   global DirList PlayList

#   puts "in append x: $x, y: $y"
   set sel [$DirList(dirlist) index @$x,$y]
   set dirname [lindex $DirList(dirname_list) $sel]
#   puts "sel = $sel, dirname = $dirname"
   lappend PlayList(playlist) "$dirname*.mp*"
   if {$PlayList(filename) == ""} {
      set t1 [file dirname $PlayList(playlist)]
#      puts "t1 = $t1"
      set t2 [file tail $t1]
#      puts "t2 = $t2"
      set PlayList(filename) $t2
   }
#   puts "PlayList = $PlayList(playlist)"
   PlayList(update)
   return
}

proc DirList(update) {dirname} {
   global FileList DirList
   global status

   # Fix beginning of dirname
   if {[string index $dirname 0] != "/"} {
      # prepend pwd
      set dirname [pwd]/$dirname
   }

   if {![check_dir $dirname]} {return}

   # Fix end of dirname
   set len [expr [string length $dirname] - 1]
   if {[string index $dirname $len] == "/"} {
      # Remove final "/"
      set dirname [string range $dirname 0 [expr $len - 1]]
   }
   # Now all dirnames are full paths with no / at the end
   # (so we can split it properly)

   $DirList(dirlist) delete 0 end
   set dir_tokens [split $dirname "/"]
   if {$dir_tokens == ""} {
      set dir_tokens "{}"
   }
   set offset ""
   set DirList(dirname_list) ""
   set last_dir ""
   foreach i $dir_tokens {
      set s $offset$i/
      $DirList(dirlist) insert end $s
      append offset "  "
      append last_dir $i/
      lappend DirList(dirname_list) $last_dir
   }

   foreach i [lsort [glob -nocomplain $dirname/* $dirname/.*]] {
      if {[file isdirectory $i]} {
         set s [file tail $i]/
         if {$s != "./" && $s != "../"} {
            $DirList(dirlist) insert end $offset$s
            lappend DirList(dirname_list) $last_dir$s
         }
      }
   }

   set len [llength $dir_tokens]
   $DirList(dirlist) selection set [expr $len - 1]
   set DirList(directory) $dirname/

   FileList(update) $dirname
   return
}

proc check_file {filename} {
   global status

   set file [glob -nocomplain $filename]
   if {[llength $file] != 1} {
      set status "$filename is not a valid file"
      return 0
   } elseif {![file isfile $filename]} {
      set status "$filename is not a valid file"
      return 0
   } else {
      set status ""
      return 1
   }
}

proc check_dir {dirname} {
   global status

   set dir [glob -nocomplain $dirname]
   if {[llength $dir] != 1} {
      set status "$dirname is not a valid directory"
      return 0
   } elseif {![file isdirectory $dirname]} {
      set status "$dirname is not a valid directory"
      return 0
   } else {
      set status ""
      return 1
   }
}

proc PlayList(clear) {} {
   global PlayList status

   if {[info exists PlayList(playlist)]} {unset PlayList(playlist)}
   set PlayList(filename) ""
   PlayList(update)
   set status ""
}

proc PlayList(remove) {} {
   global PlayList

   # Remove all selected elements from playlist
   set sel [$PlayList(listbox) curselection]
   set sorted [lsort -decreasing -integer $sel]
   foreach i $sorted {
      set PlayList(playlist) [lreplace $PlayList(playlist) $i $i]
   }
   PlayList(update)
   return
}

proc PlayList(update) {} {
   global PlayList status

   $PlayList(listbox) delete 0 end
#   puts "In playlist update: $PlayList(playlist)"
   if {[info exists PlayList(playlist)]} {
#      puts "looping"
      foreach i $PlayList(playlist) {
         set len [$PlayList(listbox) size]
         set next [expr $len + 1]
         set space "    "
         set s $next.$space$i
         $PlayList(listbox) insert end $s
      }
   }
   set status ""
   return
}

proc PlayList(append) {} {
   global PlayList
   global FileList DirList

   # Append elements from FileList
   set sel [$FileList(filelist) curselection]
   foreach i $sel {
      set file [$FileList(filelist) get $i]
      set file $DirList(directory)$file
      if {[file extension $file] == ".mp3" || 
          [file extension $file] == ".mp2" || 
          [file extension $file] == ".MP3" ||
          [file extension $file] == ".MP2"} {
         lappend PlayList(playlist) $file
      } else {
         # Ask user about playlist selection
#         set button [tk_dialog .query "Append Playlist?" \
#            "Do you want to append the contents of the playlist $file?" \
#            "" 0 Yes Cancel]
#         if {$button == 0} {
            # Load the contents of the playlist file (1 = append mode)
            PlayList(load) $file 1
#         }
      }
   }
   PlayList(update)
   $PlayList(listbox) see end
   return
}

proc PlayList(replace) { } {
   global PlayList FileList

   set numsel [llength [$FileList(filelist) curselection]]
   if {[info exists PlayList(playlist)]} {
      set button [tk_dialog .query "Replace Playlist?" \
      "Do you want to replace the playlist with the $numsel selected files?" \
         "" 0 Yes Cancel]
      if {$button != 0} {return}
   }
   PlayList(clear)
   PlayList(append)

   return
}

proc PlayList(load) {playlist {append 0}} {
   global PlayList
   global FileList
   global status

   if {!$append} {
      PlayList(clear)
   }
   set fp [open $playlist r]
   set good 1
   while {![eof $fp] && $good} {
      set playline [gets $fp]
      set length [llength $playline]
      if {$length > 1} {
         if {[scan "$playline" "%d" pause] == 1} {
	    # Pause value present
	    # Scan again to find the starting index of the filename
            if {[scan $playline "%d %s" spause string1] != 2} {
               set status "error in playlist $listfile : line = $playline"
               set good 0
            }
            set starti [string first $string1 "$playline"]
	    set filespec [string range "$playline" $starti end]
         } else {
	    # Set filespec = entire line (w/ spaces)
 	    set filespec "$playline"
         }
      } elseif {$length == 1} {
         # Set filespec = entire line (no spaces)
         set filespec "$playline"
      } else {
         # length = 0  -blank line -skip
         continue
      }

      lappend PlayList(playlist) $filespec
   }
   close $fp
   set PlayList(filename) $playlist
   PlayList(update)
   return
}

proc PlayList(save) {} {
   global PlayList PLAYDIR
   global status

   if {$PlayList(filename) == ""} {
      set status "Playlist filename is blank"
      return
   }

   if {[string index $PlayList(filename) 0] != "/"} {
      # Save to $PLAYDIR
      set PlayList(filename) $PLAYDIR/$PlayList(filename)
   }
   set fp [open $PlayList(filename) w]
   foreach i $PlayList(playlist) {puts $fp $i}
   close $fp
   set status "Playlist saved in file $PlayList(filename)"
   return
}

proc PlayList(saveas) {} {
   global PlayList
   global status
   set playlist [tk_getSaveFile -defaultextension m3u \
      -title "Save PlayList As"]
   if {$playlist != ""} {
      set status "Saving to $playlist"
      set PlayList(filename) $playlist
      PlayList(save)
   }
   return
}

proc PlayList(open) {} {
   global PlayList status
   set playlist [tk_getOpenFile -defaultextension m3u -title "Open Playlist"]
   if {$playlist != ""} {
      PlayList(load) $playlist
   }
   set status ""
   return
}

proc PlayList(apply) {} {
   global PlayList status cdir
   upvar #0 state s
   upvar #0 playlist p
   upvar #0 display d
   upvar #0 temp t

   set t(playlist_len) 0
   set t(playlist) ""

   if {![info exists PlayList(playlist)]} { return }
   foreach filespec $PlayList(playlist) {
#      puts "filespec = $filespec"
      if [IsURL "$filespec"] {
	set filelist "$filespec"
      } else {
        set filelist [lsort [glob -nocomplain "$filespec"]]
      }
      if {[llength $filelist] == 0} {
         set status "$filespec matches no files"
      } else {
#         puts "filelist = $filelist"
         foreach filename $filelist {
#            puts "filename = $filename"
            if {[file isfile "$filename"]  || [IsURL "$filename"]} {
#               puts "before playlist_Add"
               playlist_add "$filename" $s(bgpause) temp
#               puts "after playlist add"
            } else {
	       if {$filename != "."} {
                  set status "$filename is an invalid file"
               }
	    }
	 }
#         puts "foreach filename"
      }
   }

#   puts "t(playlist_len) = $t(playlist_len)"
   if {$t(playlist_len) != 0} {
      if {$s(pipe_open)} {
         set c $s(current_song)
#         puts "llength t(playlist) = [llength t(playlist)]"
         if {[llength $t(playlist)] < $c} {
            stop
            set s(current_song) 0
         }
         if {[lindex $p(playlist) $c] != [lindex $t(playlist) $c]} {
            stop
            set s(current_song) 0
         }
      }

      # Copy the temp playlist into the real one.
      set p(songlen_list) $t(songlen_list)
      set p(playlist_len) $t(playlist_len)
      set p(layer_list) $t(layer_list)
      set p(outrate_list) $t(outrate_list)
      set p(inrate_list) $t(inrate_list)
      set p(mode_list) $t(mode_list)
      set p(playlist) $t(playlist)
      set p(pause_list) $t(pause_list)
      # Clear temp playlist
      unset t(songlen_list)
      unset t(playlist_len)
      unset t(layer_list)
      unset t(outrate_list)
      unset t(inrate_list)
      unset t(mode_list)
      unset t(playlist)
      unset t(pause_list)
#      puts "near end"
      if {$PlayList(filename) != ""} {
         set d(pl_name) [string toupper [file tail $PlayList(filename)]]
      }
      if {$s(current_song) == -1} {set s(current_song) 0}
      init_display
   } else {
#      puts "Playlist $listfile contained invalid entries"
   }
   return
}

proc modal_edit { } {
   global PlayList update playline pos

   set sel [$PlayList(listbox) curselection]
   if {[llength $sel] == 0} {return}
   set orig [lindex $sel 0]
   set playline [lindex $PlayList(playlist) $orig]

   set update 0
   toplevel .edit
   wm title .edit "Edit Playlist Line"
   set pos [expr $orig + 1]
   frame .edit.1
   label .edit.1.label -text "Playlist position:"
   entry .edit.1.pos -textvariable pos -width 4
   frame .edit.2
   label .edit.2.label -text "Filename:"
   entry .edit.2.entry -textvariable playline -width 50
   button .edit.ok -text "OK" -command {set update yes}
   button .edit.cancel -text "Cancel" -command {set update no}
   pack .edit.1 -side top -fill x -expand 1
   pack .edit.1.label -side left -anchor w
   pack .edit.1.pos -side left -anchor w
   pack .edit.2 -side top -fill x -expand 1
   pack .edit.2.label -side left -anchor w
   pack .edit.2.entry -side left -anchor w -fill x
   pack .edit.ok -side left -expand 1
   pack .edit.cancel -side left -expand 1

   tkwait variable update

   # Update the playlist be deleting the current
   # entry and putting the new one in.
   if {$update == "yes"} {
      set PlayList(playlist) [lreplace $PlayList(playlist) $orig $orig]
      set new [expr $pos - 1]
      set PlayList(playlist) [linsert $PlayList(playlist) $new $playline]
   }
   destroy .edit
   PlayList(update)
   return
}

#
# Remove the window
#
proc PlayList(exit) { } {
   global .playedit cdir DirList
   upvar #0 state s

   set cdir $DirList(directory)
   destroy .playedit
   set s(playedit_open) 0
   return
}

#
# Create the window
#
proc PlayList(playedit) { } {
   global status FileList DirList FileList PlayList PlayEdit
   global PLAYDIR version cdir
   upvar #0 state s
   upvar #0 playlist p

   set FileList(fchoices) $PlayEdit(fchoices)
   set FileList(filter) "*"
   set status ""

   if {$s(playedit_open)} {return}
   if {![info exists cdir]} {
      if {[file exists $PLAYDIR]} {
         set cdir [glob $PLAYDIR]
      } else {
         set cdir [pwd]
      }
   }

   set s(playedit_open) 1

   # Create the playedit window
   toplevel .playedit
   wm title .playedit "Playlist Editor"

   # Status bar on the bottom

   label .playedit.b -textvariable status -relief sunken -anchor w
   pack .playedit.b -side bottom -fill x

   # Directory Stuff

   frame .playedit.t
   set d .playedit.t.d
   frame $d -relief raised -bd 2
   label $d.label -text "Directory:" -anchor w
   entry $d.entry -textvariable DirList(directory) -relief sunken \
      -width 40 -font $PlayEdit(entryfont) -bg $PlayEdit(bgcolor) \
      -fg $PlayEdit(fghighlight) -insertbackground $PlayEdit(crcolor)
   set DirList(entry) $d.entry
   frame $d.m -relief sunken -bd 2
   listbox $d.m.dirlist -relief sunken \
      -yscrollcommand "$d.m.scroller set" -bg $PlayEdit(bgcolor) \
      -fg $PlayEdit(fgcolor) -selectbackground $PlayEdit(bgcolor) \
      -selectforeground $PlayEdit(fghighlight) -selectborderwidth 0 \
      -font $PlayEdit(listfont)
   set DirList(dirlist) $d.m.dirlist
   scrollbar $d.m.scroller -command "$d.m.dirlist yview" -width 12

   pack .playedit.t -side top -fill both -expand 1
   pack $d -side left -fill both -expand 1
   pack $d.label -side top -fill x -anchor w
   pack $d.entry -side top -fill x -anchor w
   pack $d.m -side top -fill both -expand 1
   pack $d.m.dirlist -side left -fill both -expand 1
   pack $d.m.scroller -side left -fill y

   bind $d.entry <Key-Return> {
      if {[check_dir $DirList(directory)] == 1} {
        DirList(update) $DirList(directory) }
   }
   bind $d.m.dirlist <ButtonRelease-1> {DirList(select)}
   bind $d.m.dirlist <ButtonRelease-3> {DirList(append) %x %y}

   # File Stuff

   set f .playedit.t.f
   frame $f -relief raised -bd 2
   label $f.filelab -text "Filename:" -anchor w
   entry $f.filename -textvariable FileList(filename) -relief sunken \
      -width 45 -font $PlayEdit(entryfont) \
      -bg $PlayEdit(bgcolor) -fg $PlayEdit(fghighlight) \
      -highlightcolor $PlayEdit(bgcolor) -insertbackground $PlayEdit(crcolor)
   set FileList(entry) $f.filename
   frame $f.m -relief sunken -bd 2
   listbox $f.m.filelist -relief sunken \
      -yscrollcommand "$f.m.scroller set" -bg $PlayEdit(bgcolor) \
      -fg $PlayEdit(fgcolor) -selectbackground $PlayEdit(bgcolor) \
      -selectforeground $PlayEdit(fghighlight) -font $PlayEdit(listfont) \
      -selectborderwidth 0 -height 10 -selectmode extended
   set FileList(filelist) $f.m.filelist
   scrollbar $f.m.scroller -command "$FileList(filelist) yview" -width 12
   frame $f.ff
   label $f.ff.filtlab -text "Filter:"
   entry $f.ff.filter -textvariable FileList(filter) -relief sunken \
      -font $PlayEdit(entryfont) -bg $PlayEdit(bgcolor) \
      -fg $PlayEdit(fghighlight) -insertbackground $PlayEdit(crcolor)
   set FileList(filter_entry) $f.ff.filter
   menubutton $f.ff.choice -text "Choice" -relief raised \
      -menu $f.ff.choice.menu
   set FileList(choice) $f.ff.choice
   menu $f.ff.choice.menu
   foreach i $FileList(fchoices) {
      $f.ff.choice.menu add command -label "$i" -command \
         "set FileList(filter) $i; FileList(update) \$DirList(directory)"
   }
   pack $f -side right -fill both -expand 1
   pack $f.filelab -side top -fill x
   pack $f.filename -side top -fill x
   pack $f.ff -side bottom -fill x
   pack $f.ff.filtlab -side left -anchor w
   pack $f.ff.choice -side right
   pack $f.ff.filter -side left -fill x -expand 1
   pack $f.m -side top -fill both -expand 1 -anchor w
   pack $f.m.scroller -side right -fill y
   pack $f.m.filelist -side left -fill both -expand 1

   bind $f.filename <Key-Return> {FileList(enter)}
   bind $f.m.filelist <ButtonRelease-1> {FileList(select)}
   bind $f.m.filelist <Double-1> {PlayList(append)}
   bind $f.ff.filter <Key-Return> {FileList(update) $DirList(directory)}

   # The middle buttons

   frame .playedit.mb
   set m .playedit.mb
   button $m.clear -text "Clear" -command {PlayList(clear)}
   button $m.edit -text "Edit" -command {modal_edit}
   button $m.replace -text "Replace \\/" -command {PlayList(replace)}
   button $m.append -text "Append \\/" -command {PlayList(append)}
   button $m.remove -text "Remove /\\" -command {PlayList(remove)}
   pack $m -fill x
   pack $m.clear -side left -expand 1
   pack $m.edit -side left -expand 1
   pack $m.replace -side left -expand 1
   pack $m.append -side left -expand 1
   pack $m.remove -side left -expand 1

   # The bottom buttons

   frame .playedit.bb
   set b .playedit.bb
   button $b.open -text "Open" -command {PlayList(open)}
   button $b.save -text "Save to Disk" -command {PlayList(save)}
   button $b.saveas -text "Save As ..." -command {PlayList(saveas)}
   button $b.play -text "Save to Tk3play" -command {PlayList(apply)}
   button $b.close -text "Close" -command {PlayList(exit)}
   pack $b -side bottom -fill x
   pack $b.open -side left -expand 1
   pack $b.save -side left -expand 1
   pack $b.saveas -side left -expand 1
   pack $b.play -side left -expand 1
   pack $b.close -side left -expand 1

   # The Playlist

   frame .playedit.p -relief raised -bd 2
   set m .playedit.p
   frame $m.t
   label $m.t.label -text "Playlist:" -anchor w
   entry $m.t.filename -textvariable PlayList(filename) -relief sunken \
      -width 25 -font $PlayEdit(entryfont) \
      -bg $PlayEdit(bgcolor) -fg $PlayEdit(fghighlight) \
      -highlightcolor $PlayEdit(bgcolor) -insertbackground $PlayEdit(crcolor)
   set pl(entry) $m.t.filename
   set PlayList(filename) ""
   listbox $m.playlist -relief sunken \
      -yscrollcommand "$m.scroller set" -bg $PlayEdit(bgcolor) \
      -fg $PlayEdit(fgcolor) -selectbackground $PlayEdit(bgcolor) \
      -selectforeground $PlayEdit(fghighlight) -font $PlayEdit(listfont) \
      -selectborderwidth 0 -height 7 -selectmode extended
   set PlayList(listbox) $m.playlist
   scrollbar $m.scroller -command "$m.playlist yview" -width 12
   pack $m -side top -fill both -expand 1
   pack $m.t -side top -fill x
   pack $m.t.label -side left
   pack $m.t.filename -side left -fill x -expand 1
   pack $m.playlist -side left -fill both -expand 1
   pack $m.scroller -side right -fill y
   bind $m.playlist <Double-1> {modal_edit}

   DirList(update) $cdir

   if {[info exists p(playlist)]} {
      set PlayList(playlist) $p(playlist)
   }

   PlayList(update)

   return
}
